package top;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import database.MongoDBInteraction;
import object_detection.ObjectDetector;
import object_detection.types.ObjectSet;
import object_detection.types.Point;
import object_detection.types.PointSet;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.config.annotation.CorsRegistry;
import org.springframework.web.servlet.config.annotation.EnableWebMvc;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@SpringBootApplication
public class BackendJava {

    public static void main(String[] args) {
        SpringApplication.run(BackendJava.class, args);
    }

    @RestController
    @RequestMapping("/api")
    public static class ObjectSetController {

        private final MongoDBInteraction dbInteraction = new MongoDBInteraction();
        private final Gson gson = new GsonBuilder().create();

        @GetMapping("/{index}")
        public ResponseEntity<String> getObjectSet(@PathVariable int index) {
            try {
                ObjectSet objectSet = dbInteraction.retrieveLatestObjectSet();
                if (objectSet != null) {
                    return ResponseEntity.ok(gson.toJson(objectSet));
                } else {
                    return ResponseEntity.notFound().build();
                }
            } catch (NumberFormatException e) {
                return ResponseEntity.badRequest().body("{\"error\":\"Invalid index format\"}");
            } catch (Exception e) {
                return ResponseEntity.internalServerError().body("{\"error\":\"Internal Server Error: " + e.getMessage() + "\"}");
            }
        }

        @GetMapping("/getObjects")
        public ResponseEntity<String> getObjects() {
            try {
                ObjectSet objectSet = dbInteraction.retrieveLatestObjectSet();

                List<String> indexes = new ArrayList<>();
                for(PointSet p : objectSet.objects){
                    indexes.add(Integer.toString(p.getIDX()));
                }

                Map<String, List<String>> result = new HashMap<>();
                result.put("objects", indexes);

                if (!indexes.isEmpty()) {
                    return ResponseEntity.ok(gson.toJson(result));
                } else {
                    return ResponseEntity.notFound().build();
                }
            } catch (Exception e) {
                return ResponseEntity.internalServerError().body("{\"error\":\"Failed to retrieve all objects: " + e.getMessage() + "\"}");
            }
        }
    }

    @Controller
    public static class BackendService {

        private final MongoDBInteraction dbInteraction = new MongoDBInteraction();
        private final Gson gson = new GsonBuilder().create();

        @RequestMapping("/hello")
        @ResponseBody
        public String hello() {
            return "Hello There";
        }

        @RequestMapping("/")
        public String index() {
            return "html/index";
        }

        @RequestMapping("/runProcess")
        @ResponseBody
        public boolean runProcess() throws FileNotFoundException {
            System.out.println(" ============> Starting process");
            ObjectDetector.startProcess();
            return true;
        }

        @RequestMapping("/getJSON")
        @ResponseBody
        public ResponseEntity<String> tempJson() {
            try {
                ObjectSet latestObjectSet = dbInteraction.retrieveLatestObjectSet();

                Map<String, PointSet> result = new HashMap<>();
                for(PointSet p : latestObjectSet.objects){
                    result.put(Integer.toString(p.getIDX()), p);
                }

                if (!result.isEmpty()) {
                    return ResponseEntity.ok(gson.toJson(result));
                } else {
                    return ResponseEntity.notFound().build();
                }
            } catch (Exception e) {
                return ResponseEntity.internalServerError().body("{\"error\":\"Error retrieving latest object set: " + e.getMessage() + "\"}");
            }
        }

        @RequestMapping("/style/main.css")
        public String getStyle() {
            return "style/main.css";
        }

        @RequestMapping("/js/app.js")
        public String getApp() {
            return "js/app.js";
        }

        @RequestMapping("/js/pointCloud.js")
        public String getPC() {
            return "js/pointCloud.js";
        }
    }

    @Configuration
    @EnableWebMvc
    public static class WebMvcConfig implements WebMvcConfigurer {
        @Override
        public void addCorsMappings(CorsRegistry registry) {
            registry.addMapping("/**").allowedOrigins("http://localhost:5555");
        }
    }
}
